package xyz.mcutils;

import xyz.mcutils.common.WebRequest;
import xyz.mcutils.exception.ErrorResponse;
import xyz.mcutils.models.mojang.CachedMojangEndpointStatus;
import xyz.mcutils.models.player.CachedPlayer;
import xyz.mcutils.models.player.CachedPlayerSkinPart;
import xyz.mcutils.models.player.CachedUsernameToUuid;
import xyz.mcutils.models.player.Skin;
import xyz.mcutils.models.server.*;

public class McUtilsAPI {
    private static final String API_ENDPOINT = "https://api.mcutils.xyz";
    private static final String MOJANG_API_STATUS_ENDPOINT = API_ENDPOINT + "/mojang/status";
    private static final String PLAYER_ENDPOINT = API_ENDPOINT + "/player/%s";
    private static final String USERNAME_TO_UUID_ENDPOINT = API_ENDPOINT + "/player/uuid/%s";
    private static final String PLAYER_SKIN_PART_ENDPOINT = API_ENDPOINT + "/player/%s/%s";
    private static final String SERVER_ENDPOINT = API_ENDPOINT + "/server/%s/%s";
    private static final String SERVER_ICON_ENDPOINT = API_ENDPOINT + "/server/icon/%s";
    private static final String SERVER_BLOCKED_STATUS_ENDPOINT = API_ENDPOINT + "/server/blocked/%s";

    /**
     * Gets the status of the Mojang APIs.
     *
     * @return The status of the Mojang API.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedMojangEndpointStatus getMojangApiStatus() throws ErrorResponse {
        return WebRequest.get(MOJANG_API_STATUS_ENDPOINT, CachedMojangEndpointStatus.class);
    }


    /**
     * Gets a player from the API.
     *
     * @param id The id of the player to get.
     * @return The player.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedPlayer getPlayer(String id) throws ErrorResponse {
        return WebRequest.get(PLAYER_ENDPOINT.formatted(id), CachedPlayer.class);
    }

    /**
     * Gets a player from the API.
     *
     * @param username The username of the player to get.
     * @return The player.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedUsernameToUuid getUsernameToUuid(String username) throws ErrorResponse {
        return WebRequest.get(USERNAME_TO_UUID_ENDPOINT.formatted(username), CachedUsernameToUuid.class);
    }

    /**
     * Gets a player skin part from the API.
     *
     * @param part The part of the skin to get.
     * @param id The id of the player to get.
     * @return The player skin part.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedPlayerSkinPart getPlayerSkinPart(Skin.SkinPart part, String id) throws ErrorResponse {
        byte[] partBytes = WebRequest.get(PLAYER_SKIN_PART_ENDPOINT.formatted(part, id), byte[].class);
        return new CachedPlayerSkinPart(partBytes);
    }

    /**
     * Gets a Bedrock server from the API.
     *
     * @param id The id of the server to get.
     * @return The server.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedBedrockMinecraftServer getBedrockServer(String id) throws ErrorResponse {
        return WebRequest.get(SERVER_ENDPOINT.formatted(ServerPlatform.BEDROCK.name(), id), CachedBedrockMinecraftServer.class);
    }

    /**
     * Gets a Java server from the API.
     *
     * @param id The id of the server to get.
     * @return The server.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedJavaMinecraftServer getJavaServer(String id) throws ErrorResponse {
        return WebRequest.get(SERVER_ENDPOINT.formatted(ServerPlatform.JAVA.name(), id), CachedJavaMinecraftServer.class);
    }

    /**
     * Gets the icon of a Java server from the API.
     *
     * @param id The id of the server to get the icon of.
     * @return The server icon.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedServerIcon getServerIcon(String id) throws ErrorResponse {
        byte[] bytes = WebRequest.get(SERVER_ICON_ENDPOINT.formatted(id), byte[].class);
        return new CachedServerIcon(bytes);
    }

    /**
     * Gets the blocked status of a Java server from the API.
     *
     * @param id The id of the server to get the blocked status of.
     * @return The server blocked status.
     * @throws ErrorResponse If an error occurs.
     */
    public static CachedServerBlockedStatus getServerBlockedStatus(String id) throws ErrorResponse {
        return WebRequest.get(SERVER_BLOCKED_STATUS_ENDPOINT.formatted(id), CachedServerBlockedStatus.class);
    }
}